"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ScratchGrid = void 0;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/*
    Fastest way to calculate percentage of rect area covered by unknown shape is downsampling.
    A (cell width = brush stroke diameter) is the best choice.
    Having bigger cells means in each cell more area remains uncleared when cell is marked.
    Having smaller cells can result in unmarked cells on cleared part of the screen.
*/
class ScratchGrid {
  constructor(rectSize, lineWidth) {
    _defineProperty(this, "percentCompleted", 0);

    _defineProperty(this, "size", void 0);

    _defineProperty(this, "grid", void 0);

    _defineProperty(this, "totalCellsCount", void 0);

    _defineProperty(this, "cellWidth", void 0);

    _defineProperty(this, "markedCellsCount", 0);

    const numberOfElementsInRow = Math.ceil(rectSize.width / lineWidth); // x

    const rowsCount = Math.ceil(rectSize.height / lineWidth); // y

    this.grid = Array(rowsCount).fill([]).map(() => Array(numberOfElementsInRow).fill(false));
    this.totalCellsCount = rowsCount * numberOfElementsInRow;
    this.cellWidth = lineWidth;
    this.size = rectSize;
  }

  update(scratchPoint) {
    if (this.isCellScratched(scratchPoint)) {
      this.markedCellsCount += 1;
    }

    this.percentCompleted = this.markedCellsCount / this.totalCellsCount * 100;

    if (__DEV__) {
      // For Debug Console autoscroll in VSCode, uncheck Settings > Debug > Console: Word Wrap
      for (let row of this.grid) {
        console.log(row.map(isMarked => isMarked ? '*' : ' '));
      }

      console.log(' ');
    }
  }

  isCellScratched(brush) {
    const rowIndex = Math.floor(brush.y / this.cellWidth);
    const elementInRowIndex = Math.floor(brush.x / this.cellWidth);
    const cell = this.grid[rowIndex][elementInRowIndex];

    if (!cell) {
      this.grid[rowIndex][elementInRowIndex] = true;
      return true;
    }

    return false;
  }

}

exports.ScratchGrid = ScratchGrid;
//# sourceMappingURL=ScratchGrid.js.map